#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
import types
import os
import logging
import logging.config
import smtplib
from email.mime.base      import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text      import MIMEText
from email.utils          import COMMASPACE
from email                import Encoders
import time
import ConfigParser
import sys
import base64

class EmailSender(object):
    SENDMAIL_MAX_RETRY = 3

    def __init__(self, smtp_server_addr, smtp_server_port, is_ehlo=False, is_ssl=False, is_auth=False, auth_password=''):
        self.smtp_server_addr = smtp_server_addr
        self.smtp_server_port = smtp_server_port

        try:
            config_file = "/etc/iscan/intscan.ini"
            config = ConfigParser.SafeConfigParser()
            config.read(config_file)

            is_ehlo = config.get('notification', 'use_ehlo', False)
            if is_ehlo == 'yes':
                is_ehlo = True
            else:
                is_ehlo = False

            is_ssl = config.get('notification', 'use_ssl', False)
            if is_ssl == 'yes':
                is_ssl = True
            else:
                is_ssl = False

            is_auth = config.get('notification', 'use_auth', False)
            if is_auth == 'yes':
                is_auth = True
            else:
                is_auth = False

            auth_password = config.get('notification', 'auth_password', '')

        except Exception, e:
            logging.exception('failed to get config: %s', e)
            is_ehlo = False
            is_ssl = False
            is_auth = False
            auth_password = ''

        self.is_ehlo = is_ehlo 
        self.is_ssl = is_ssl
        self.is_auth = is_auth
        self.auth_password = auth_password

    def send_email(self, from_addr,             # sender address
                         to_addr,               # receiver addresses
                         subject,               # subject of email
                         text,                  # text content
                         html = None,           # html content
                         attach_file = [],      # attached files list
                         file_display_name = [],# file display name
                         reply_addr = None):    # none
        try:
            logging.debug('from_addr=%s, to_addr=%s, subject=%s, text=%s, attach_file=%s, file_display_name=%s, reply_addr=%s, html=%s',
                           from_addr, to_addr, subject, text, attach_file, file_display_name, reply_addr, html)

            if file_display_name and len(file_display_name) != len(attach_file):
                logging.error('length of attach file(%s) and display name(%s) is not same' , attach_file, file_display_name)
                logging.error('length of attach file(%d) and display name(%d) is not same' , len(attach_file), len(file_display_name))
                return False

            message = self._make_message(from_addr, to_addr, subject, text, html, attach_file, file_display_name, reply_addr)

            logging.info('connect to SMTP server - ip(%s),port(%s)' , self.smtp_server_addr, self.smtp_server_port)
            client = smtplib.SMTP(self.smtp_server_addr, self.smtp_server_port)
            client.set_debuglevel(0)
        except Exception, e:
            logging.exception('exception happens when making prameters, error code:%s' , e)
            return False

        logging.info('sending email')
        
        for i in xrange(self.SENDMAIL_MAX_RETRY):
            try:
                self._sendmail(client, from_addr, to_addr, message.as_string())
                #client.sendmail(from_addr, to_addr, message.as_string())
                result = True
                break
            except Exception, e:
                logging.exception('sending mail exception, error code: %s' , e)
                result = False
                time.sleep(3)
        client.quit()
        return result

    def _sendmail(self, client, from_addr, to_addr, msg):
        if self.is_ehlo:
            client.ehlo() 

        if self.is_auth:
            if self.is_ssl:
                client.starttls()
            client.login(from_addr, self.auth_password)

        client.sendmail(from_addr, to_addr, msg)
        

    def _make_message(self, from_addr, to_addr, subject, text, html = None, attach_file = [], file_display_name = [], reply_addr = None):
        message = MIMEMultipart('mixed')
        message.set_charset('UTF-8')

        message['Subject'] = self._encode_str(subject)
        message['From']    = self._encode_str(from_addr)
        message['To']      = self._convert_to_strings(to_addr)
        if reply_addr:
            message['Reply-To'] = self._convert_to_strings(reply_addr)
        
        attachfile = ''
        attachfilename = ''
        body = ''

        if subject.find('DLP Alert') != -1:
            text, attachfile, attachfilename = getAttachfile(text)
            file_display_name.append(attachfilename)
            attach_file.append(attachfile)

        message.attach(MIMEText(self._encode_str(text), 'plain', 'utf8'))

        for index, filename in enumerate(attach_file):
            file_part = MIMEBase('application', "octet-stream")
            file_part.set_payload(open(filename,"rb").read())
            Encoders.encode_base64(file_part)
            if file_display_name:
                file_part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(file_display_name[index]))
            else:
                file_part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(filename))
            message.attach(file_part)

        if html:
            message.attach(MIMEText(self._encode_str(html), 'html', 'utf8'))
        
        if attachfile:
            os.remove(attachfile)

        return message

    def _convert_to_strings(self, list_of_strs):
        if isinstance(list_of_strs, (list, tuple)):
            result = COMMASPACE.join(list_of_strs)
        else:
            result = list_of_strs
        return self._encode_str(result)

    def _encode_str(self, s):
        if type(s) == types.UnicodeType:
            s = s.encode('utf8')
        return s

def test_send(smtp_server, port, from_addr, to_addr, is_ehlo, is_ssl, is_auth, auth_password):
    test_subject = 'test mail'
    test_body = 'test mail body'
    print 'smtp server: ' + smtp_server + ':' + str(port)
    print 'begin send test mail to ' + to_addr + ' from ' + from_addr
    print 'is_ehlo: ' + str(is_ehlo)
    print 'is_ssl: ' + str(is_ssl)
    print 'is_auth: ' + str(is_auth)
    print 'auth_password: ' + str(auth_password)
    print 'test subject: ' + str(test_subject)
    print 'test body: ' + str(test_body)

    test_obj = EmailSender(smtp_server, port, is_ehlo, is_ssl, is_auth, auth_password)
    test_obj.send_email(from_addr, to_addr, test_subject, test_body)

def init_logging():
    try:
        logging_config_file = '/usr/iwss/schedule_report/config/email_sender_logging.ini' 
        logging.config.fileConfig(logging_config_file)
        logging.debug('init_logging finished')
    except Exception, e:
        logging.exception('failed to init logging: %s', e)
        exit(1)

def getAttachfile(textBody):
    attachfilename = ''
    attachfile = ''
    file = ''
    body = ''
    flag = 0
    attchfilebase64 = 0
    bodyindex = 0
    for line in textBody.splitlines():
        if line.find("NextPart_000-0000-AAAAAAAA-AAAAAAAA") != -1:
            flag = 0
        elif line.find("text/plain") != -1:
            flag = 1
        elif line.find("attachment;") != -1:
            bodyindex = 1
            line1 = line.find("\"")
            line2 = line.find("\"",line1+1)
            if line1 == -1 or line2 == -1:
                logging.error('attachfile error')
                return False
            attachfilename = line[line1+1:line2]
            flag = 2

        if flag == 1:
            if bodyindex ==1:
                bodyindex = bodyindex + 1
            else:
                body += line
        elif flag == 2:
            if attchfilebase64 != 2:
                attchfilebase64 += 1
            else:
                attachfile += line
    if attachfilename:
        file = "/var/iwss/mailq/" + attachfilename
        logging.debug('filename: %s', file)
        file_ob = open(file, 'w')
        file_ob.write(base64.b64decode(attachfile))
        file_ob.close()
    return body, file, attachfilename
    

def read_email_file(email_file):
    try:
       logging.info('read email content from file: %s', email_file)
       to_addr_list = []
       subject = ''
       body = ''
       with open(email_file) as email_fp:
           while 1:
               line = email_fp.readline()
               if not line:
                   break
               if line.startswith('T '):
                   to_addr = line.replace('T ', '') 
                   to_addr = to_addr.replace('\n', '') 
                   to_addr_list.append(to_addr)
                   continue
               if line.startswith('S '):
                   subject = line.replace('S ', '') 
                   subject = subject.replace('\n', '') 
                   continue
               if line.startswith('B '):
                   tmp_line = line.replace('B ', '')
                   if tmp_line:
                       logging.info('get one line of body: %s', line)
                       tmp_line = tmp_line.replace('\\n', '\n')
                       body += tmp_line
                   else:
                       pass
                   continue

           return to_addr_list, subject, body
    except Exception, e:
        logging.exception('failed to read email file: %s, %s', email_file, e)
        exit(1)

if __name__ == '__main__':
    #read configuration
    try:
        config_file = "/etc/iscan/intscan.ini"
        config = ConfigParser.SafeConfigParser()
        config.read(config_file)

        smtp_server = config.get('notification', 'server', 'localhost')
        port = config.getint('notification', 'port')

        is_ehlo = config.get('notification', 'use_ehlo', False)
        if is_ehlo == 'yes':
            is_ehlo = True
        else:
            is_ehlo = False

        is_ssl = config.get('notification', 'use_ssl', False)
        if is_ssl == 'yes':
            is_ssl = True
        else:
            is_ssl = False

        is_auth = config.get('notification', 'use_auth', False)
        if is_auth == 'yes':
            is_auth = True
        else:
            is_auth = False

        auth_password = config.get('notification', 'auth_password', '')

        from_addr = config.get('notification', 'from_addr')
        to_addr = config.get('notification', 'to_addr')
    except:
        print 'failed to get configuration'
        exit(1)
    
    #do test function
    if len(sys.argv) == 1:
        test_send(smtp_server, port, from_addr, to_addr, is_ehlo, is_ssl, is_auth, auth_password)
        exit(0)

    #init logging
    init_logging()

    #read mail content from file
    if len(sys.argv) != 2:
        logging.info('wrong argc')
        exit(1)

    to_addr_list, subject, body = read_email_file(sys.argv[1])
    return_value = 0
    for to_addr in to_addr_list:
        mail_obj = EmailSender(smtp_server, port, is_ehlo, is_ssl, is_auth, auth_password)
        if mail_obj.send_email(from_addr, to_addr, subject, body):
            continue
        else:
            return_value = 1

    exit(return_value)
    
